/**
 * Alibaba.com Inc.
 * Copyright (c) 2004-2016 All Rights Reserved.
 */
package com.aliyun.iot.demo;

import java.io.IOException;

import com.aliyun.iot.CcpClient;
import com.aliyun.iot.CcpMessageListener;
import com.aliyun.iot.connect.ConnectOpts;
import com.aliyun.iot.message.AbstractMessage;
import com.aliyun.iot.message.ConnectACK;
import com.aliyun.iot.message.PubAck;
import com.aliyun.iot.message.Publish;
import com.aliyun.iot.message.RRequest;
import com.aliyun.iot.message.RResponse;
import com.aliyun.iot.message.Request;
import com.aliyun.iot.message.StatusCode;
import com.aliyun.iot.message.Subscribe;
import com.aliyun.iot.message.UnSubscribe;
import com.aliyun.iot.util.LogUtil;

/**
 * 这个demo描述一个设备如何连接云端进行数据上报
 * 
 * <p>
 * 具体说明
 * </p>
 * 
 * @author
 * @version $Id: IotClientDemo.java,v 0.1 2016年1月19日 下午3:58:00 Exp $
 */
public class IotClientDemo {

    static ConnectOpts opts = new ConnectOpts();

    /*************************************************************************
     * 运行前请先在Config.java里面设置您的设备信息 设备连接到云端需要3个过程：
     * *********************************************************************** 
     * (1)调用设备配置接口初始化设备，服务器返回公钥以及数据网关服务器地址
     * (2)生成您自己的私钥seedkey，并使用服务器公钥加密发给服务器，成功后得到sessionId （sid）
     * (3)使用sid连接数据网关（即第一步返回的服务器地址），进行数据传输
     * ***********************************************************************
     * 
     * @param args
     * @throws Exception
     *************************************************************************/
    public static void main(String[] args) throws Exception {
        
        
        // 以下参数请到控制台申请 http://iot.console.aliyun.com/iot/
        opts.setDeviceName("xxxx");// 设备名称
        opts.setDeviceSecret("xxxx");// 设备密钥
        opts.setProductAppKey("xxxx");// 产品证书key
        opts.setProductAppSecret("xxxx");// 产品证书私钥
        opts.setAuthUrl("http://iot.channel.aliyun.com");// 认证地址

        // 以下参数可选
        opts.setAliveSeconds((short) 80);// 心跳时间
        opts.setSignMethod("HmacMD5");// HmacMD5 HmacSh1 Md5
        opts.setAutoReconnect(true);// 是否自动重连
        opts.setAutoReconnectDelay(5000);// 重连间隔时间

        // 建立ccp会话通信
        final CcpClient ccpClient = new CcpClient(opts);
        // 把当前实例增加为监听对象，异步消息会回调到监听对象
        ccpClient.attachListener(new CcpMessageListener() {
            @Override
            public void onMessage(AbstractMessage message) {
                LogUtil.print("收到消息：" + message);
                
                
                
                //这段demo是针对 rrpc协议，如果是服务器发送rrequest，客户端需要返回rresponse               
//                                if(message instanceof RRequest){
//                                    RResponse resp = new RResponse();
//                                    resp.sequenceID = message.sequenceID;//这里很重要，要保证回去的和请求的一致，服务器才能认为是同一个东西
//                                    resp.payload = "返回内容".getBytes();
//                                    resp.statusCode = StatusCode.success;
//                                    ccpClient.sendMessageNoWait(resp,false);
//                                    
//                                }

            }

            @Override
            public void onConnect(ConnectACK message) {
                LogUtil.print("连接消息：" + message);
                if (message.statusCode == StatusCode.success) {
                    testSubPub(ccpClient);
                    testRPC(ccpClient);
                }
            }

            @Override
            public void onOffLine() {
                LogUtil.print("会话断开");
            }

        });

        // 连接数据服务器
        ccpClient.connect();

        // ///block
        try {
            System.in.read();
        } catch (IOException ex) {
            ex.printStackTrace();
        }

    }

    /**
     * 测试数据上报
     * 
     * @param s
     * @throws InterruptedException
     */
    private static void testRPC(CcpClient s) {
        Request msg5 = new Request();
        msg5.platformId = 2;
        msg5.sid = s.getSessionId();
        msg5.apiType = 3;

        msg5.contentType = 2;
        msg5.resourceType = 2;
        msg5.resourceUrl = opts.getProductAppKey() + ";" + opts.getProductAppKey() + ";1";

        msg5.body = "这个是上报给自己服务器的内容 ".getBytes();

        //异步方式调用，消息通过 onMessage 监听
        s.sendMessageNoWait(msg5);

        //同步方式调用，等待服务器结果回执
        //Response result = s.requestAndWait(msg5);

    }

    /**
     * 测试sub pub消息
     * 
     * @param s
     */
    private static void testSubPub(final CcpClient s) {
        String topic = "/" + opts.getProductAppKey() + "/b";
        try {

            LogUtil.print("5秒后模拟一个sub、再5秒发送pub、再15秒发送unsub、5秒后再尝试pub");
            // sub 和 pub 使用时候需要使用topic，请到管理控制台->设备管理 进行授权
            Thread.sleep(5000);
            LogUtil.print("5秒后 ，模拟发送一个数据sub请求，订阅的队列=" + topic);
            Subscribe msg = new Subscribe();
            msg.topicFilters = new String[] { topic };
            s.sendMessageNoWait(msg);

            Thread.sleep(5000);
            LogUtil.print("5秒后 ，模拟发送一个数据pub请求，发布到队列=" + topic);

            Publish msg2 = new Publish();
            msg2.topic = topic;
            msg2.aliveSecond = 30;
            msg2.payload = "{tem:20}".getBytes();
            //同步方式调用
            PubAck result = s.publish(msg2);
            LogUtil.print("publish result=" + result);

            Thread.sleep(5000);
            LogUtil.print("5秒后 ，模拟发送一个unsub请求，取消队列=" + topic);
            UnSubscribe msg3 = new UnSubscribe();
            msg3.topicFilters = new String[] { topic };
            //异步方式调用
            s.sendMessageNoWait(msg3);

            Thread.sleep(5000);
            msg2.sequenceID = 0;
            LogUtil.print("5秒后 ，再发送一个pub消息，队列=" + topic);
            result = s.publish(msg2);
            LogUtil.print("publish result=" + result);

        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

}
