/**
 * Alibaba.com Inc.
 * Copyright (c) 2004-2016 All Rights Reserved.
 */
package com.aliyun.iot.handler;

import com.aliyun.iot.message.AbstractMessage;
import com.aliyun.iot.message.ConnectACK;
import com.aliyun.iot.message.MessageHeader;
import com.aliyun.iot.message.MessagePacket;
import com.aliyun.iot.message.PingRequest;
import com.aliyun.iot.message.PubAck;
import com.aliyun.iot.message.Publish;
import com.aliyun.iot.message.Push;
import com.aliyun.iot.message.RRequest;
import com.aliyun.iot.message.ReconnectACK;
import com.aliyun.iot.message.Request;
import com.aliyun.iot.message.Response;
import com.aliyun.iot.message.SubAck;
import com.aliyun.iot.message.UnSubAck;
import com.aliyun.iot.message.UnknownSession;
import com.aliyun.iot.util.LogUtil;

/**
 * 消息执行器
 *
 * <p>具体说明</p>
 *
 * @version $Id: CcpMessagehandler.java,v 0.1 2016年1月20日 下午6:53:17  Exp $
 */
public class CcpMessagehandler {
    /** seedKey  */
    private byte[] seedKey;

    public CcpMessagehandler(byte[] seedkey) {
        this.seedKey = seedkey;
    }

    public AbstractMessage hander(MessageHeader header, byte[] payload) throws Exception {

        LogUtil.print("handler msg：" + header);

        //1，构造消息实例
        AbstractMessage ackmsg = null;

        if (header.messageType == MessagePacket.CONNECT_ACK) {
            //回执消息
            ackmsg = new ConnectACK(header);
        } else if (header.messageType == MessagePacket.SUBACK) {
            //新增订阅消息的回执
            ackmsg = new SubAck(header);
        } else if (header.messageType == MessagePacket.PUBACK) {
            //////发送消息的回执
            ackmsg = new PubAck(header);
        } else if (header.messageType == MessagePacket.PUBLISH) {
            //收到服务器端推送的pub消息
            ackmsg = new Publish(header);
        } else if (header.messageType == MessagePacket.PINGREQUEST) {
            ackmsg = new PingRequest(header);
        } else if (header.messageType == MessagePacket.REQUEST) {
            ackmsg = new Request(header);
        } else if (header.messageType == MessagePacket.UNSUBACK) {
            ackmsg = new UnSubAck(header);
        } else if (header.messageType == MessagePacket.RESPONSE) {
            ackmsg = new Response(header);
        } else if (header.messageType == MessagePacket.PUSH) {
            ackmsg = new Push(header);
        } else if (header.messageType == MessagePacket.R_REQUEST) {
            ackmsg = new RRequest(header);
        } else if(header.messageType == MessagePacket.UNKNOWN_SESSION){
            ackmsg = new UnknownSession(header);
        }else if(header.messageType == MessagePacket.RECONNECT_ACK){
            ackmsg = new ReconnectACK(header);
        }else{
            throw new RuntimeException("unsurported message");
        }

        //2，解析消息
        if (ackmsg != null) {
            ackmsg.decode(payload, seedKey);
        }

        return ackmsg;

    }

    /**
     * Getter method for property <tt>seedKey</tt>.
     * 
     * @return property value of seedKey
     */
    public byte[] getSeedKey() {
        return seedKey;
    }

    /**
     * Setter method for property <tt>seedKey</tt>.
     * 
     * @param seedKey value to be assigned to property seedKey
     */
    public void setSeedKey(byte[] seedKey) {
        this.seedKey = seedKey;
    }

}
