/**
 * Alibaba.com Inc.
 * Copyright (c) 2004-2016 All Rights Reserved.
 */
package com.aliyun.iot.handler;

import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;

import com.aliyun.iot.internal.Session;
import com.aliyun.iot.message.MessageHeader;
import com.aliyun.iot.message.StatusCode;
import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.LogUtil;

/**
 * 用一个线程 不断的接收消息，每收到一个消息，交给处理器
 *
 * 简单实现
 *
 * @version $Id: RecieveMessageThread.java,v 0.1 2016年1月20日 下午7:09:15  Exp $
 */
public class RecieveMessageThread extends Thread {

    private Socket    client  = null;
    private Session session = null;
    private boolean   stop    = false;

    public RecieveMessageThread(Socket client, Session s) {
        this.client = client;
        this.session = s;
    }

    public void run() {
        try {
            InputStream in = client.getInputStream();
            while (!stop) {
                byte[] hb = new byte[20 * 1024];
                int len = in.read(hb);
                if (len <= 0) {
                    Thread.sleep(300);
                    continue;
                }

                byte[] data = new byte[len];
                System.arraycopy(hb, 0, data, 0, len);
                handlePack(data);
            }
        } catch (IOException e) {
            e.printStackTrace();
            session.setStatusCode(StatusCode.offline);
            session.notifyOffline();
        } catch (Exception e) {
            e.printStackTrace();
        }

        LogUtil.print("thread end");
    }

    ///////以下简单处理一个拆包和解包问题//////

    DynamicByteBuffer append      = new DynamicByteBuffer(500, false);
    boolean           hasHead     = false;
    long              packWishLen = 0;
    byte[]            payload     = null;
    MessageHeader     header      = null;

    private void handlePack(byte[] pack) throws Exception {
        append.put(pack);

        if (!hasHead) {
            //1，解析包头
            DynamicByteBuffer buffer = new DynamicByteBuffer(pack);
            header = new MessageHeader(buffer);
            LogUtil.print("from server: " + header);
            if (!header.hasData) {
                //有的消息没有内容
                session.onMessage(header, null);
                append.clear();
                hasHead = false;
                return;
            }
            payload = new byte[(int) header.remainLength];
            packWishLen = header.remainLength + header.index();
            if (header.remainLength == -1) {
                //包头不完整
                header = new MessageHeader(append);
                payload = new byte[(int) header.remainLength];
                packWishLen = header.remainLength + header.index();
            }
        }

        if (header.remainLength == -1) {
            return;
        }

        byte[] curentBytes = append.array();

        //解析消息
        if (packWishLen == curentBytes.length) {
            System.arraycopy(curentBytes, header.index(), payload, 0,
                (int) packWishLen - header.index());
            //有了一个完整消息
            session.onMessage(header, payload);
            append.clear();
            hasHead = false;
        } else if (packWishLen < curentBytes.length) {
            //消息在一个包内,有多余
            System.arraycopy(curentBytes, header.index(), payload, 0,
                (int) packWishLen - header.index());
            //有了一个完整消息
            session.onMessage(header, payload);
            append.clear();

            byte[] byteslast = new byte[(int) (curentBytes.length - packWishLen)];
            System.arraycopy(curentBytes, (int) packWishLen, byteslast, 0,
                (int) (curentBytes.length - packWishLen));
            //剩余部分
            hasHead = false;
            handlePack(byteslast);

        } else if (packWishLen > curentBytes.length) {
            //当前包内容不足，等待追加
            hasHead = true;
        } else {
            //error，不应该这里
            LogUtil.print("packet process error");
        }

    }

    /**
     * Getter method for property <tt>stop</tt>.
     * 
     * @return property value of stop
     */
    public boolean isStop() {
        this.interrupt();
        return stop;
    }

    /**
     * Setter method for property <tt>stop</tt>.
     * 
     * @param stop value to be assigned to property stop
     */
    public void setStop(boolean stop) {
        this.stop = stop;
    }
    
    

}
