/**
 * Alibaba.com Inc.
 * Copyright (c) 2004-2016 All Rights Reserved.
 */
package com.aliyun.iot.internal;

import java.util.Collection;
import java.util.Hashtable;

import com.aliyun.iot.message.AbstractMessage;
import com.aliyun.iot.message.MessagePacket;

/**
 * 锁列表
 *
 * @version $Id: TokenStore.java,v 0.1 2016年6月28日 上午10:13:37  Exp $
 */
public class TokenStore {

    /** lock list **/
    private Hashtable<String, SynRespToken> tokens = new Hashtable<String, SynRespToken>();

    /**
     * create token
     * @param key
     * @return
     */
    public SynRespToken createToken(AbstractMessage msg) {
        String key = SynRespToken.generateKey(msg);
        SynRespToken token = new SynRespToken(key);
        tokens.put(key, token);
        return token;
    }

    /**
     * get token with key
     * 
     * @param key
     * @return
     */
    public SynRespToken getToken(AbstractMessage msg) {
        //有限的类型需要 ， 只有客户端发起的请求对应返回结果有可能存在同步等待 
        switch (msg.getMessageType()) {
            case MessagePacket.PUBACK:
            case MessagePacket.SUBACK:
            case MessagePacket.RESPONSE:
            case MessagePacket.CONNECT_ACK:
            case MessagePacket.RECONNECT_ACK:
            case MessagePacket.UNKNOWN_SESSION:
            case MessagePacket.UNSUBACK:
                String key = SynRespToken.generateKey(msg);
                return tokens.get(key);
            default:
                return null;
        }
    }

    /**
     *remove token with key
     * 
     * @param key
     * @return
     */
    public SynRespToken removeToken(AbstractMessage msg) {
        String key = SynRespToken.generateKey(msg);
        return tokens.remove(key);
    }

    /**
     * remove all
     */
    public void shutDown() {
        if (tokens.size() > 0) {
            Collection<SynRespToken> ts = tokens.values();
            for (SynRespToken t : ts) {
                t.notifyComplete(true);
            }
        }
        tokens.clear();
    }

}
