package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;


/**
 * Created by Gary on 14-5-4. 响应用户创建连接的请求
 */
public class ConnectACK extends AbstractMessage {

    public StatusCode statusCode = StatusCode.success; // 连接返回的状态码

    public String     connectionToken;                // 链接成功后，由服务端返回的会话令牌，在Recoonect时使用。（在重连失败后为空）
    public String     suggestionIPs;                  // 服务端返回给客户端的最新的推荐服务器IP列表（可为空，通常在用户地域发生变化时会返回）
    public short      keepAlive  = 240;               // 心跳时间，单位秒
    public String     latestSDKPath;                  // 推荐版本更新地址（不为空）
    public int        appID;                          // 使用推送服务的应用授权ID，在应用申请时由系统统一生成

    public ConnectACK(){

        super(CONNECT_ACK);
    }

    public ConnectACK(MessageHeader fixedHeader){

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(256);

        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // StatusCode
        data.put((byte) this.statusCode.code());
        // Connection Token
        data.put(ProtocolUtils.encodeString(this.connectionToken));
        // Suggestion IPs
        data.put(ProtocolUtils.encodeString(this.suggestionIPs));
        // Keep Alive
        data.putShort(this.keepAlive);
        // latestSDKPath
        data.put(ProtocolUtils.encodeString(latestSDKPath));
        // App ID
        ProtocolUtils.encodeVariableNumber(data, this.appID);

        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {

        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // StatusCode
        this.statusCode = StatusCode.from(buffer.get());
        // Connection Token
        this.connectionToken = ProtocolUtils.decodeString(buffer);
        // Suggestion IPs
        this.suggestionIPs = ProtocolUtils.decodeString(buffer);
        // Keep Alive
        this.keepAlive = buffer.getShort();
        // latestSDKPath
        this.latestSDKPath = ProtocolUtils.decodeString(buffer);
        //AppID
        this.appID = (int)ProtocolUtils.decodeVariableNumber(buffer);
    }

    @Override
    public String toString() {
        return super.toString() + "; ConnectACK{" + "statusCode=" + statusCode + ", connectionToken='"
               + connectionToken + '\'' + ", keepAlive='" + keepAlive + '\'' + ",latestSDKPath:"
               + latestSDKPath + '}';
    }
}
