package com.aliyun.iot.message;

import com.aliyun.iot.util.CipherUtils;
import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.EncryptType;
import com.aliyun.iot.util.ProtocolUtils;


public class ConnectExtAuth extends AbstractMessage {

    public static final byte encrypt_lowest_version = 19;

    public byte version = 20; // 版本号
    public byte network; // 客户端的网络信息（高4位：网络类型；低4位：运营商类型）
    public String appAccount; // 应用内部标识一个账户的唯一字符串。可用来配合实现应用级别的按用户推送
    public String packageName; // 应用的packageName
    public int platformId;
    public String sid; // 经过外部认证握手后生成的唯一sid
    public short limit = 50; // 最大的离线消息 限制
    public short keepAlive;// 单位秒，客户端的心跳时间建议值（服务端有权修改），小于或等于60表示由服务端决定(服务端通过ConnectAck返回).

    public transient byte[] payload;// 高版本要加解密，暂时保存

    public ConnectExtAuth() {

        super(CONNECT_EXT_AUTH);
    }

    public ConnectExtAuth(MessageHeader header) {

        super(header);
    }

    @Override
    public byte[] encodeBody() {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(300);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // Version
        data.put(this.version);

        if (this.version < encrypt_lowest_version) {// 为了兼容，低于19版本不做加解密
            // AppAccount
            data.put(ProtocolUtils.encodeString(this.appAccount));
            // PackageName
            data.put(ProtocolUtils.encodeString(this.packageName));
            // Network
            data.put(this.network);
            // Platform ID
            ProtocolUtils.encodeVariableNumber(data, this.platformId);
            // SID
            data.put(ProtocolUtils.encodeString(this.sid));
            // limit
            ProtocolUtils.encodeVariableNumber(data, this.limit);
            // keepAlive
            ProtocolUtils.encodeVariableNumber(data, this.keepAlive);
        } else {
            // Platform ID
            ProtocolUtils.encodeVariableNumber(data, this.platformId);
            // SID
            data.put(ProtocolUtils.encodeString(this.sid));
            // 对称 加密的结果
            data.put(payload);
        }
        
//        this.fixedHeader.remainLength = data.array().length;
//        DynamicByteBuffer result = DynamicByteBuffer.allocate(300);
//        result.put(this.fixedHeader.generateFixedHeader());
//        result.put(data.array());
        return data.array();
    }

    /**
     * 在数据Encode时，必须先调用这个方法进行数据加密（因为服务端的SeedKey是与链接相关联的）
     * 
     * @param seedKey
     */
    public void encryptPayload(byte[] seedKey) {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(80);

        // Network
        data.put(this.network);
        // appAccount
        data.put(ProtocolUtils.encodeString(this.appAccount));
        // PackageName
        data.put(ProtocolUtils.encodeString(this.packageName));
        // limit
        ProtocolUtils.encodeVariableNumber(data, limit);
        // keepAlive
        ProtocolUtils.encodeVariableNumber(data, keepAlive);
        this.payload = CipherUtils.aesEncrypt(data.array(), seedKey);
    }
    
    /**
     * 解密payload
     * @param seedKey
     * @throws Exception 
     */
    public void decryptPayload(byte[] seedKey) throws Exception {
        byte[] data = CipherUtils.aesDecrypt(this.payload, seedKey);
        DynamicByteBuffer buffer = new DynamicByteBuffer(data);

        // Network
        this.network = buffer.get();
        // appAccount
        this.appAccount = ProtocolUtils.decodeString(buffer);
        // PackageName
        this.packageName = ProtocolUtils.decodeString(buffer);
        // limit
        this.limit = (short) ProtocolUtils.decodeVariableNumber(buffer);
        // keepAlive
        this.keepAlive = (short) ProtocolUtils.decodeVariableNumber(buffer);
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer)  {

        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // Version
        this.version = buffer.get();

        if (this.version < encrypt_lowest_version) {// 为了兼容，低于19版本不做加解密
            // AppAccount
            this.appAccount = ProtocolUtils.decodeString(buffer);
            // PackageName
            this.packageName = ProtocolUtils.decodeString(buffer);
            // Network
            this.network = buffer.get();
            // Platform ID
            this.platformId = (int) ProtocolUtils.decodeVariableNumber(buffer);
            // SID
            this.sid = ProtocolUtils.decodeString(buffer);
            // limit
            if (this.version >= 12 && buffer.hasRemaining()) {// 版本兼容逻辑
                this.limit = (short) ProtocolUtils.decodeVariableNumber(buffer);
                if (buffer.hasRemaining()) {
                    this.keepAlive = (short) ProtocolUtils.decodeVariableNumber(buffer);
                }
            }
        } else {
            // platformId
            this.platformId = (int) ProtocolUtils.decodeVariableNumber(buffer);
            // SID
            this.sid = ProtocolUtils.decodeString(buffer);
            // 没解密的 payload
            this.payload = new byte[buffer.remaining()];
            buffer.get(this.payload);
        }
    }

   

    public EncryptType getEncryptType() {

        return EncryptType.noNeed;
    }

    @Override
    public String toString() {
        return super.toString() + "; ConnectExtAuth{" + "version=" + version + ", network="
                +  ", appAccount=" + appAccount + ",packageName:" + packageName
                + ",platformId:" + platformId + ",sid:" + sid + ",limit:" + limit + ",keepAlive:" + keepAlive + "}";
    }

}
