package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.InvalidDataPacket;
import com.aliyun.iot.util.ProtocolUtils;


/**
 * Created by Gary on 14-5-6.
 */
public class MessageHeader {

    public int messageType = -1; // 消息类型
    public boolean compress; // 是否压缩
    public int qosLevel = 1; // 消息可靠级别，只有0和1两个值,0代表最多只有一次传递，1代表至少有一次传递
    public boolean hasData = true; // 是否需要加解密

    public long remainLength; // 包括可变头以及Payload的总长度

    public DynamicByteBuffer buffer;

    public MessageHeader(DynamicByteBuffer buffer) throws InvalidDataPacket {

        this.buffer = buffer;

        this.decodeFixHead();
    }

    public MessageHeader(int messageType) {

        if (messageType > 31) {

            throw new IllegalArgumentException(" message type is big than 31");
        }

        this.messageType = messageType;
    }

    public MessageHeader(int messageType, boolean compress, int qosLevel, boolean hasData, int remainLength) {

        this(messageType);

        this.compress = compress;
        this.qosLevel = qosLevel;
        this.hasData = hasData;
        this.remainLength = remainLength;
    }

    /**
     * 生成固定头（包括remaining length）
     * 
     * @return
     */
    public byte[] generateFixedHeader() {

        byte header = 0;

        if (hasData) {

            header |= 0x01;
        }

        if (this.qosLevel == 1) {

            header |= 0x02;
        }

        if (this.compress) {

            header |= 0x04;
        }

        // Fixed header
        header = (byte) (this.messageType << 3 | header);

        // Remaining Length
        if (this.remainLength > 0) {

            if (this.remainLength < 1024 * 1024) {

                byte[] remainLengthValues = this.encodeRemainLength();

                byte[] result = new byte[remainLengthValues.length + 1];
                result[0] = header;
                System.arraycopy(remainLengthValues, 0, result, 1, remainLengthValues.length);

                return result;
            } else {

                throw new IllegalArgumentException("remain length is great than 1 Mb");
            }
        }

        return new byte[] { header };
    }

    /**
     * 解码固定头
     * 
     * @throws InvalidDataPacket
     */
    public boolean decodeFixHead() throws InvalidDataPacket {

        if (buffer.hasRemaining()) {

            byte header = buffer.get();

            // 解码Header
            this.messageType = (header & 0x00F8) >> 3;

            if (!(this.messageType >= 0 && this.messageType < 32)) {

                throw new InvalidDataPacket(InvalidDataPacket.UNKNOW_PACKET);
            }

            this.hasData = (header & 0x01) == 1;
            this.qosLevel = (header & 0x02) >> 1;
            this.compress = ((header & 0x04) >> 2) == 1;

            if (this.hasData && buffer.remaining() > 2) {

                // decodeBody remaining length
                this.remainLength = this.decodeRemainLength();
            }
            return true;
        }else{
            return false;
        }
    }

    private byte[] encodeRemainLength() {

        return ProtocolUtils.encodeVariableNumber(this.remainLength);
    }

    protected long decodeRemainLength() {

        return ProtocolUtils.decodeVariableNumber(buffer);
    }
    
    /**
     * 当前buffer索引
     * 
     * @return
     */
    public int index() {

        return buffer.position();
    }


    @Override
    public String toString() {

        return "FixedHeader{" + "remainLength=" + remainLength + ", messageType=" + messageType + ", compress="
                + compress + ", qosLevel=" + qosLevel + ", hasData=" + hasData + '}';
    }
}
