package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.EncryptType;
import com.aliyun.iot.util.InvalidDataPacket;

/**
 * Created by Gary on 14-5-8.
 */
public interface MessagePacket {

    /**
     * 协议当前版本
     */
    public static byte VERSION = 21;

    public static final int PACKET_SIZE_LIMIT = 1 * 1024 * 1024;
    /**
     * 默认的最大的离线消息拉取
     */
    public static final byte DEFAULT_LIMIT = 50;
    // ======================================================
    // 协议消息类型常量
    // ======================================================
    public static final byte RESERVED_0 = 0;

    public static final byte CONNECT = 1;
    public static final byte CONNECT_ACK = 2;
    public static final byte PUSH = 3;
    public static final byte PUSH_ACK = 4;
    public static final byte PINGREQUEST = 5;
    public static final byte PINGREQUEST_ACK = 6;
    public static final byte DISCONNECT = 7;
    public static final byte ATTACH = 8;
    public static final byte ATTACH_ACK = 9;
    public static final byte DETACH = 10;
    public static final byte DETACH_ACK = 11;
    public static final byte RECONNECT = 12;
    public static final byte RECONNECT_ACK = 13;
    public static final byte INNER_REQUEST = 14;
    public static final byte INNER_RESPONSE = 15;
    public static final byte UNKNOWN_SESSION = 16;
    public static final byte REQUEST = 17;
    public static final byte RESPONSE = 18;
    public static final byte CONNECT_EXT_AUTH = 19;
    public static final byte ATTACH_EXT_AUTH = 20;
    public static final byte FETCH = 21;
    public static final byte FETCH_ACK = 22;
    /** 反向RPC,由服务端来调用客户端，要求客户端返回调用的结果,从20协议版本开始支持 add by xiaosong **/
    public static final byte R_REQUEST = 23;
    public static final byte R_RESPONSE = 24;
    /** pub/sub的支持 add by xiaosong **/
    // PUBLISH
    public static final byte PUBLISH = 25;
    // PUBACK
    public static final byte PUBACK = 26;
    // SUBSCRIBE
    public static final byte SUBSCRIBE = 27;
    // SUBACK
    public static final byte SUBACK = 28;
    // UNSUBSCRIBE
    public static final byte UNSUBSCRIBE = 29;
    // UNSUBACK
    public static final byte UNSUBACK = 30;
    // ======================================================

    // 一次最多订阅的topic数
    public final static int TOPIC_LIMIT = 64;

    public byte[] encodeBody();

    public void decodeBody(DynamicByteBuffer buffer)  throws InvalidDataPacket;

    public EncryptType getEncryptType();
}
