package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * publish 指令 client <---> server,A PUBLISH Control Packet is sent from a Client to a Server or from Server to a Client
 * to transport an Application Message. 支持QoS,0和1 ，0代表最多只有一次传递，1代表至少有一次传递
 * pub sub 是基于队列的
 */
public class Publish extends AbstractMessage {

    public String topic; // topic name
    public int aliveSecond;// 需要ACK的消息(至少发送一次的消息)存活的时间，如果该值为0，则说明不需要保存（至多发送一次的消息）
    public byte[] payload; // payload

    public Publish() {
        super(PUBLISH);
    }

    public Publish(MessageHeader header) {
        super(header);
    }

    @Override
    public byte[] encodeBody() {
        DynamicByteBuffer data = DynamicByteBuffer.allocate(256);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // topic name
        data.put(ProtocolUtils.encodeString(topic));
        // aliveSecond
        ProtocolUtils.encodeVariableNumber(data, this.aliveSecond);
        // payload
        data.put(payload);
        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {
        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // topic name
        this.topic = ProtocolUtils.decodeString(buffer);
        // aliveSecond
        this.aliveSecond = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // payload
        this.payload = new byte[buffer.remaining()];
        buffer.get(payload);
    }

    @Override
    public String toString() {
        return "Publish[sequenceID:+" + sequenceID + ",topic:" + topic + ",aliveSecond:" + aliveSecond
                + ",payload.length:" + (payload == null ? 0 : payload.length) + "]，paload转字符串：" + new String(payload);
    }
}
