package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * 这个是由服务器主动根据设备id推送消息给客户端
 * Created by Gary on 14-5-5.
 */
public class Push extends AbstractMessage {

    public long   messageID; // 消息唯一标识符
    public int    appID;    // 应用的唯一标识
    public byte[] content;  // 推送的内容

    public Push() {

        super(PUSH);

        this.setLastFrame(true);
    }

    public Push(MessageHeader header) {
        super(header);
    }

    @Override
    public byte[] encodeBody() {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(512);

        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // Message ID
        ProtocolUtils.encodeVariableNumber(data, this.messageID);
        // App ID
        ProtocolUtils.encodeVariableNumber(data, this.appID);
        // 推送的内容
        data.put(this.content);

        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {

        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // Device ID
        this.messageID = ProtocolUtils.decodeVariableNumber(buffer);
        // App ID
        this.appID = (int) ProtocolUtils.decodeVariableNumber(buffer);

        // 推送的内容
        this.content = new byte[buffer.remaining()];
        buffer.get(this.content);
    }

    @Override
    public String toString() {

        return "Push{" + "messageID=" + messageID + ", appID=" + appID + ", content="
               + (content == null ? "null" : content.length) + '}'
               + ((content == null || content.length == 0) ? "" : new String(content));
    }
}
