package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * 这个是推送方式，由服务器主动根据设备id推送消息给客户端,客户端应该发送的回执
 * Created by Gary on 14-5-5.
 */
public class PushACK extends AbstractMessage {

    public long messageID;
    public int  appID;
    public byte type = 3; // 消息接收是3，打开是4,删除是8

    public PushACK() {

        super(PUSH_ACK);
    }

    public PushACK(MessageHeader fixedHeader) {

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(128);

        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // Message ID
        ProtocolUtils.encodeVariableNumber(data, this.messageID);
        // App ID
        ProtocolUtils.encodeVariableNumber(data, this.appID);
        // type
        data.put(type);
        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {

        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // Message ID
        this.messageID = ProtocolUtils.decodeVariableNumber(buffer);
        // App ID
        this.appID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // type
        this.type = buffer.get();
    }

    @Override
    public String toString() {
        return super.toString() + "; PushACK{" + "messageID='" + messageID + '\'' + ", appID='"
               + appID + '\'' + ",type=" + type + '}';
    }
}
