package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

public class RRequest extends AbstractMessage {

    public int appID; // 应用的唯一标识

    public byte[] payload; // 向客户端请求的内容

    public RRequest() {
        super(R_REQUEST);
    }

    public RRequest(MessageHeader fixedHeader) {

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {
        DynamicByteBuffer data = DynamicByteBuffer.allocate(1024);
        // 写入sequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // 写入appId
        ProtocolUtils.encodeVariableNumber(data, this.appID);
        // 写入 payload长度
        ProtocolUtils.encodeVariableNumber(data, payload == null ? 0 : payload.length);
        if (payload != null && payload.length > 0) {
            data.put(payload);
        }
        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer){
        // 读出sequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // 读出appId
        this.appID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // 读出rpc payload
        int len = (int) ProtocolUtils.decodeVariableNumber(buffer);
        if (len > PACKET_SIZE_LIMIT || len > buffer.remaining()) {
            throw new RuntimeException("the rrequest payload's size:" + len + " is exceed " + PACKET_SIZE_LIMIT
                    + " or buffer.remaining():" + buffer.remaining());
        }
        byte[] bytes = new byte[len];
        buffer.get(bytes);
        this.payload = bytes;
    }

    @Override
    public String toString() {
        return "RRequest [appID=" + appID + ",sequenceID:" + sequenceID + ", payload.length:"
                + (payload == null ? 0 : payload.length) + "]"+new String(payload);
    }
}
