package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

public class RResponse extends AbstractMessage {

    public StatusCode statusCode = StatusCode.success;

    public byte[] payload;// 返回给服务端响应结果或错误消息

    public RResponse() {
        super(R_RESPONSE);
    }

    public RResponse(MessageHeader fixedHeader) {

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(1024);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // 写入statusCode
        data.put((byte) this.statusCode.code());
        // 写入payload
        ProtocolUtils.encodeVariableNumber(data, payload == null ? 0 : payload.length);
        if (payload != null && payload.length > 0) {
            data.put(payload);
        }
        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer)  {
        // 读出sequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // 读出statusCode
        this.statusCode = StatusCode.from(buffer.get());
        // 读出rpc payload
        int len = (int) ProtocolUtils.decodeVariableNumber(buffer);
        if (len > PACKET_SIZE_LIMIT || len > buffer.remaining()) {
            throw new RuntimeException("the rresponse payload's size:" + len + " is exceed " + PACKET_SIZE_LIMIT
                    + " or buffer.remaining():" + buffer.remaining());
        }
        byte[] bytes = new byte[len];
        buffer.get(bytes);
        this.payload = bytes;
    }

    @Override
    public String toString() {
        return "RResponse [statusCode=" + statusCode + ", payload.length:" + (payload == null ? 0 : payload.length)
                + "]";
    }

}
