package com.aliyun.iot.message;


import com.aliyun.iot.connect.ConnectOpts;
import com.aliyun.iot.util.CipherUtils;
import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.EncryptType;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * Created by Gary on 14-5-5.
 */
public class Reconnect extends AbstractMessage {

    public byte version = VERSION; // 版本号
    public String deviceID; // 设备的唯一标识
    public String connectionToken; // 正常登录时由服务端返回的授权Token，作为快速重联的认证凭证
    public byte ipSwitch; // 0为没有切换, , 1为连接的IP发生了切换
    public byte network; // 客户端的网络信息（高4位：网络类型；低4位：运营商类型）
    public short limit = DEFAULT_LIMIT; // 离线消息条数的推送的最大 限制
    public short keepAlive;// 单位秒，客户端的心跳时间建议值（服务端有权修改），小于或等于60表示由服务端决定(服务端通过ReconnectAck返回).

    private byte[] payload;
    
    private ConnectOpts opts;

    public Reconnect(ConnectOpts opts) {
        super(RECONNECT);
        this.opts = opts;
    }

    public Reconnect(MessageHeader fixedHeader) {

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {
        
        DynamicByteBuffer data = DynamicByteBuffer.allocate(128);
        data.put(this.version);
        // ==========================================================================================
        // Device ID
        // ==========================================================================================
        try {
            
            byte[] device = this.deviceID.getBytes();
            byte[] encryptDeviceID = CipherUtils.rsaEncrypt(device, opts.getPkey());
            data.put((byte) encryptDeviceID.length);
            data.put(encryptDeviceID);
            // IP Switch Flag
            data.put(this.ipSwitch);

        } catch (Exception e) {

            throw new RuntimeException(e);
        }

        // ==========================================================================================
        // 编码AES加密的数据段
        // ==========================================================================================
        data.put(this.payload);

        return data.array();
    }
    
    
    /**
     * 在数据Encode时，必须先调用这个方法进行数据加密（因为服务端的SeedKey是与链接相关联的）
     * 
     * @param seedKey
     */
    public void encryptPayload(byte[] seedKey) {
        DynamicByteBuffer data = DynamicByteBuffer.allocate(80);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // Network
        data.put(this.network);
        // Connection Token
        data.put(ProtocolUtils.encodeString(this.connectionToken));
        // limit
        ProtocolUtils.encodeVariableNumber(data, limit);
        // keepAlive
        ProtocolUtils.encodeVariableNumber(data, keepAlive);
        this.payload = CipherUtils.aesEncrypt(data.array(), seedKey);
    }

    

   
    

    public EncryptType getEncryptType() {

        return EncryptType.noNeed;
    }

    @Override
    public String toString() {
        return "Reconnect{" + "deviceID='" + deviceID + '\'' + ", connectionToken='" + connectionToken + '\''
                + ",ipSwitch:" + ipSwitch + ",network:" + network + ",version:" + version + ",keepAlive:" + keepAlive
                + "}";
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {
        // TODO Auto-generated method stub
        
    }
}
