package com.aliyun.iot.message;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * Created by Gary on 14-5-5.
 */
public class ReconnectACK extends AbstractMessage {

    public StatusCode statusCode = StatusCode.reject_invalid_token; // 连接返回的状态码

    public String     connectionToken;                             // 由服务端返回的一个全新的快速重连凭证（在重连失败后为空）

    public short      keepAlive  = 240;                            // 心跳时间，单位秒

    public ReconnectACK(){

        super(RECONNECT_ACK);
    }

    public ReconnectACK(MessageHeader fixedHeader){

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {

        DynamicByteBuffer data = DynamicByteBuffer.allocate(64);

        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        // StatusCode
        data.put((byte) this.statusCode.code());
        // Connection Token
        data.put(ProtocolUtils.encodeString(this.connectionToken));

        data.putShort(this.keepAlive);

        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer)  {

        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        // StatusCode
        this.statusCode = StatusCode.from(buffer.get());
        // Connection Token
        this.connectionToken = ProtocolUtils.decodeString(buffer);
        if (buffer.hasRemaining()) {
            this.keepAlive = buffer.getShort();
        }
    }

    @Override
    public String toString() {
        return super.toString() + "; ReconnectACK{" + "statusCode=" + statusCode + ", connectionToken='"
               + connectionToken + ", keepAlive=" + keepAlive + '}';
    }
}
