package com.aliyun.iot.message;

import java.util.Map;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.HeaderUtils;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * 
 * request只能由客户端发起，服务器端业务执行完毕后返回 response消息
 *
 * @author jianwen.ljw
 * @version $Id: Request.java,v 0.1 2016年1月26日 上午10:53:28  Exp $
 */
public class Request extends AbstractMessage {

    // rpc协议版本
    public short               version      = 2; // 长连接的RPC版本,1为第一个版本

    public String              sid;             // sid，安全认证的唯一标识

    public int                 platformId;      // 平台ID，百川、阿里云

    public byte                apiType      = 3; // 2015.5.27 新增属性，1为云通道自己的API，2为二方的API，3为三方的API

    //public byte[]              payload;

    ///payload的对应元素
    public byte                resourceType = 2;
    public String              resourceUrl;
    public byte                contentType = 2;
    public Map<String, Object> headers;
    public byte[]              body;

    public Request() {
        super(REQUEST);
    }

    public Request(MessageHeader fixedHeader) {

        super(fixedHeader);
    }

    @Override
    public byte[] encodeBody() {
        DynamicByteBuffer data = DynamicByteBuffer.allocate(1024);

        // 写入sequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);

        // 写入rpcVersion
        ProtocolUtils.encodeVariableNumber(data, this.version);

        // 写入platformID
        ProtocolUtils.encodeVariableNumber(data, this.platformId);

        // 写入sid
        data.put(ProtocolUtils.encodeString(this.sid));
        if (this.version > 1) {
            data.put(apiType);
        }
        // 写入rpc payload
        //data.put(payload);

        //////////////////////以下是rpc所需属性///////////////////////////

        data.put(resourceType);
        if (resourceType > 0) {
            data.put(ProtocolUtils.encodeString(this.resourceUrl));
        } else {
            //id方式已不支持
        }
        data.put(contentType);
        if (headers != null && headers.size() > 0) {
            data.put(HeaderUtils.encodeHeaders(headers));
        }else{
            byte h = 0;
            data.put(h);
        }
        data.put(body);

        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {
        // do nothing
    }

    @Override
    public String toString() {
        return "Request [version=" + version + ",sequenceID:" + sequenceID + ",platformId:"
               + platformId + ",sid=" + sid + ",apiType:" + apiType + ",resourceType="
               + resourceType + ",resourceUrl=" + resourceUrl + ",contentType=" + contentType + "]";
    }
}
