package com.aliyun.iot.message;

import java.util.Arrays;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * suback 指令 server ---> client,A SUBACK Packet is sent by the Server to the Client to confirm receipt and processing of
 * a SUBSCRIBE Packet.
 */
public class SubAck extends AbstractMessage {

    public byte[] codes;// 0成功，4代表没有权限 ，8代表未知错误
    public String[] msgs;// 比mqtt增加的

    public SubAck() {
        super(SUBACK);
    }

    public SubAck(MessageHeader header) {
        super(header);
    }

    @Override
    public byte[] encodeBody() {
        if (codes == null || codes.length == 0 || codes.length > TOPIC_LIMIT) {
            throw new IllegalArgumentException("codes:" + Arrays.toString(codes) + " is illegal!");
        }
        if (msgs != null && codes.length != msgs.length) {
            throw new IllegalArgumentException("codes.length:" + codes.length + " != msgs.length:" + codes.length);
        }
        DynamicByteBuffer data = DynamicByteBuffer.allocate(256);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);

        data.put((byte) codes.length);// 长度
        for (byte code : codes) {
            data.put(code);
        }
        if (msgs != null) {
            for (String msg : msgs) {
                data.put(ProtocolUtils.encodeString(msg));
            }
        }
        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {
        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        byte len = buffer.get();
        if (len > TOPIC_LIMIT) {
            throw new IllegalArgumentException(" codes length:" + len + " exceed limit:" + TOPIC_LIMIT);
        }
        int n = 0;
        this.codes = new byte[len];
        while (buffer.hasRemaining() && n < len) {
            this.codes[n] = buffer.get();
            n++;
        }
        if (buffer.hasRemaining()) {
            n = 0;
            this.msgs = new String[len];
            while (buffer.hasRemaining() && n < len) {
                this.msgs[n] = ProtocolUtils.decodeString(buffer);
                n++;
            }
        }
    }

    @Override
    public String toString() {
        return "SubAck[sequenceID:+" + sequenceID + ",codes:" + Arrays.toString(codes) + ",msgs:"
                + Arrays.toString(msgs) + "]";
    }
}
