package com.aliyun.iot.message;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * subscribe 指令 client ---> server,The SUBSCRIBE Packet is sent from the Client to the Server to create one or more
 * Subscriptions. 暂时不支持一条链接上支持多个APP
 */
public class Subscribe extends AbstractMessage {

    public String[] topicFilters; // topics or topic filters

    public Subscribe() {
        super(SUBSCRIBE);
    }

    public Subscribe(MessageHeader header) {
        super(header);
    }

    @Override
    public byte[] encodeBody() {
        if (topicFilters == null || topicFilters.length == 0 || topicFilters.length > TOPIC_LIMIT) {
            throw new IllegalArgumentException("topicFilters:" + Arrays.toString(topicFilters)
                                               + " is illegal!");
        }
        DynamicByteBuffer data = DynamicByteBuffer.allocate(256);
        
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        
        // topic个数
        data.put((byte) topicFilters.length);
        
        //topic encode
        for (String topic : topicFilters) {
            data.put(ProtocolUtils.encodeString(topic));
        }

        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {
        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        byte len = buffer.get();
        if (len > TOPIC_LIMIT) {
            throw new IllegalArgumentException(" topic length:" + len + " exceed limit:"
                                               + TOPIC_LIMIT);
        }
        List<String> list = new ArrayList<String>(len);
        int i = 0;
        while (buffer.hasRemaining() && i < len) {
            list.add(ProtocolUtils.decodeString(buffer));
            i++;
        }
        this.topicFilters = list.toArray(new String[list.size()]);
    }

    @Override
    public String toString() {
        return "Subscribe[sequenceID:+" + sequenceID + ",topicFilters:"
               + Arrays.toString(topicFilters) + "]";
    }
}
