package com.aliyun.iot.message;

import java.util.Arrays;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * UnSubAck 指令 server ---> client,The UNSUBACK Packet is sent by the Server to the Client to confirm receipt of an
 * UNSUBSCRIBE Packet.
 * 
 * of a SUBSCRIBE Packet.
 */
public class UnSubAck extends AbstractMessage {

    public byte[] codes;

    // public String[] msgs;

    public UnSubAck() {
        super(UNSUBACK);
    }

    public UnSubAck(MessageHeader header) {
        super(header);
    }

    @Override
    public byte[] encodeBody() {

        if (codes == null || codes.length == 0 || codes.length > TOPIC_LIMIT) {
            throw new IllegalArgumentException("codes:" + Arrays.toString(codes) + " is illegal!");
        }
        DynamicByteBuffer data = DynamicByteBuffer.allocate(256);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        data.put((byte) codes.length);// 长度
        for (byte code : codes) {
            data.put(code);
        }
        return data.array();

    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer) {

        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        byte len = buffer.get();
        if (len > TOPIC_LIMIT) {
            throw new IllegalArgumentException(" codes length:" + len + " exceed limit:" + TOPIC_LIMIT);
        }
        int n = 0;
        this.codes = new byte[len];
        while (buffer.hasRemaining() && n < len) {
            this.codes[n] = buffer.get();
            n++;
        }

    }

    @Override
    public String toString() {
        return "UnSubAck[sequenceID:+" + sequenceID + ",codes:" + Arrays.toString(codes) + "]";
    }
}
