package com.aliyun.iot.message;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.aliyun.iot.util.DynamicByteBuffer;
import com.aliyun.iot.util.ProtocolUtils;

/**
 * unSubscribe 指令 client ---> server,An UNSUBSCRIBE Packet is sent by the Client to the Server, to unsubscribe from
 * topics.
 */
public class UnSubscribe extends AbstractMessage {

    public String[] topicFilters;// topic filters

    public UnSubscribe() {
        super(UNSUBSCRIBE);
    }

    public UnSubscribe(MessageHeader header) {
        super(header);
    }

    @Override
    public byte[] encodeBody() {
        if (topicFilters == null || topicFilters.length == 0 || topicFilters.length > TOPIC_LIMIT) {
            throw new IllegalArgumentException("topicFilters:" + Arrays.toString(topicFilters) + " is illegal!");
        }
        DynamicByteBuffer data = DynamicByteBuffer.allocate(256);
        // SequenceID
        ProtocolUtils.encodeVariableNumber(data, this.sequenceID);
        data.put((byte) topicFilters.length);// 长度
        for (String topic : topicFilters) {
            data.put(ProtocolUtils.encodeString(topic));
        }
        return data.array();
    }

    @Override
    public void decodeBody(DynamicByteBuffer buffer)  {
        // SequenceID
        this.sequenceID = (int) ProtocolUtils.decodeVariableNumber(buffer);
        byte len = buffer.get();
        if (len > TOPIC_LIMIT) {
            throw new IllegalArgumentException(" topic length:" + len + " exceed limit:" + TOPIC_LIMIT);
        }
        List<String> list = new ArrayList<String>(len);
        int i = 0;
        while (buffer.hasRemaining() && i < len) {
            list.add(ProtocolUtils.decodeString(buffer));
            i++;
        }
        this.topicFilters = list.toArray(new String[list.size()]);
    }

    @Override
    public String toString() {
        return "UnSubscribe[sequenceID:+" + sequenceID + ",topicFilters:" + Arrays.toString(topicFilters) + "]";
    }
}
