package com.aliyun.iot.util;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

public class HeaderUtils {

    private static String charset = "UTF-8";

    /**
     * headers编码
     * 
     * @param headers
     * @return
     */
    public static byte[] encodeHeaders(Map<String, Object> headers) {
        DynamicByteBuffer buffer = DynamicByteBuffer.allocate(8);
        if (headers == null) {
            return new byte[1];
        }// end if

        int headerLength = 0;
        for (Map.Entry<String, Object> header : headers.entrySet()) {
            byte[] b = encodeHeader(header);
            if (b != null) {
                buffer.put(b);
                headerLength = headerLength + b.length;
            }// end if
        }// end for

        // 设置header结尾标示符
        byte end = 0;
        buffer.put(end);
        headerLength = headerLength + 1;

        byte[] headerBytes = new byte[headerLength];
        buffer.rewind();
        buffer.get(headerBytes);

        return headerBytes;
    }

    public static byte[] encodeHeader(Map.Entry<String, Object> header) {
        DynamicByteBuffer buffer = DynamicByteBuffer.allocate(8);
        int headerLength = 0;
        String key = header.getKey();
        int keyLength = 0;
        try {
            byte[] keyBytes = key.getBytes(charset);
            keyLength = (byte) keyBytes.length;
            buffer.put((byte) keyLength);
            buffer.put(keyBytes);
        } catch (UnsupportedEncodingException e) {
            return null;
        }

        headerLength = 1 + keyLength;

        Object value = header.getValue();
        int valueLength = 0;
        if (value instanceof Integer) {
            // 整形标示 251
            valueLength = 251;
            buffer.put((byte) valueLength);
            int valueInt = (Integer) value;
            buffer.put((byte) ((valueInt & 0xFF000000) >> 24));
            buffer.put((byte) ((valueInt & 0xFF0000) >> 16));
            buffer.put((byte) ((valueInt & 0xFF00) >> 8));
            buffer.put((byte) (valueInt & 0x00FF));

            headerLength = headerLength + 1 + 4;
        } else if (value instanceof Boolean) {
            // 布尔型标示252
            valueLength = (byte) 252;
            buffer.put((byte) valueLength);
            buffer.put((byte) (((Boolean) value) ? 0x01 : 0x00));

            headerLength = headerLength + 1 + 1;
        } else {
            // 字符串类型
            try {
                byte[] valueBytes = ((String) value).getBytes(charset);
                valueLength = valueBytes.length;
                buffer.put((byte) valueLength);
                buffer.put(valueBytes);

                headerLength = headerLength + 1 + valueLength;
            } catch (UnsupportedEncodingException e) {
                return null;
            }
        }// end if

        byte[] headerByte = new byte[headerLength];
        buffer.rewind();
        buffer.get(headerByte);
        return headerByte;
    }

    public static Map<String, Object> decodeHeaders(DynamicByteBuffer buffer) {
        Map<String, Object> headers = new HashMap<String, Object>();

        while (true) {
            Map<String, Object> header = decodeHeader(buffer);

            if (header != null) {
                headers.putAll(header);
            } else {
                break;
            }
        }// end while

        return headers;
    }

    public static Map<String, Object> decodeHeader(DynamicByteBuffer buffer) {
        if (buffer == null || !buffer.hasRemaining()){
            return null;
        }
        
        Map<String, Object> header = new HashMap<String, Object>();
        String key = null;
        Object value = null;
        byte keyLength = buffer.get();

        if (keyLength == 0) {
            // 说明header已经结束
            return null;
        }// end if

        byte[] keyBytes = new byte[keyLength];

        for (int i = 0; i < keyLength; i++) {
            keyBytes[i] = buffer.get();
        }// end for

        try {
            key = new String(keyBytes, charset);
        } catch (UnsupportedEncodingException e) {
            return null;
        }

        byte valueLength = buffer.get();
        switch (valueLength) {
            case (byte) 251:// int
                value = buffer.getInt();
                break;
            case (byte) 252:// boolean
                int bool = (int) buffer.get();
                value = bool == 0 ? false : true;
                break;
            default:// string
                byte[] valueBytes = new byte[valueLength];

                for (int i = 0; i < valueLength; i++) {
                    valueBytes[i] = buffer.get();
                }// end for

                try {
                    value = new String(valueBytes, charset);
                } catch (UnsupportedEncodingException e) {
                    return null;
                }
        }// end
        header.put(key, value);

        return header;
    }
}
